#ifndef POWERPLUGIN_H
#define POWERPLUGIN_H

#include <QObject>
#include <QSettings>

#include <dde-dock/dockconstants.h>
#include <dde-dock/dockplugininterface.h>
#include <dde-dock/dockpluginproxyinterface.h>

#include "dbus/dbuspower.h"

class QLabel;
class PowerPlugin : public QObject, public DockPluginInterface
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID "org.deepin.Dock.PluginInterface" FILE "dde-dock-power-plugin.json")
    Q_INTERFACES(DockPluginInterface)

public:
    PowerPlugin();
    ~PowerPlugin() Q_DECL_OVERRIDE;

    void init(DockPluginProxyInterface *proxy) Q_DECL_OVERRIDE;

    QString getPluginName() Q_DECL_OVERRIDE;

    QStringList ids() Q_DECL_OVERRIDE;
    QString getName(QString id) Q_DECL_OVERRIDE;
    QString getTitle(QString id) Q_DECL_OVERRIDE;
    QString getCommand(QString id) Q_DECL_OVERRIDE;
    QPixmap getIcon(QString) Q_DECL_OVERRIDE;
    bool configurable(const QString &id) Q_DECL_OVERRIDE;
    bool enabled(const QString &id) Q_DECL_OVERRIDE;
    void setEnabled(const QString &id, bool enable) Q_DECL_OVERRIDE;
    QWidget * getItem(QString id) Q_DECL_OVERRIDE;
    QWidget * getApplet(QString id) Q_DECL_OVERRIDE;
    void changeMode(Dock::DockMode newMode, Dock::DockMode oldMode) Q_DECL_OVERRIDE;

    QString getMenuContent(QString id) Q_DECL_OVERRIDE;
    void invokeMenuItem(QString id, QString itemId, bool checked) Q_DECL_OVERRIDE;

private slots:
    void updateIcon();

private:
    void initSettings();
    void onEnabledChanged();
    void setMode(Dock::DockMode mode);
    QString settingEnabledKey();
    QString getBatteryIcon(int percentage, bool plugged, bool symbolic = false);

private:
    QLabel * m_label;
    DockPluginProxyInterface * m_proxy;
    com::deepin::daemon::DBusPower * m_dbusPower;
    Dock::DockMode m_mode = Dock::FashionMode;
    QSettings * m_settings;
    QTimer *m_detectionDBusTimer = NULL;
};

#endif // POWERPLUGIN_H
